/*
FwCalendar: Monthly Calendar for FiveWin.

Sintaxis: FwCalendar([dDate] [,nRow,nCol [,oDlg]) --> dNewDate

Parmetros:

   dDate: Optional.- Date Field or variable. Defaults Date().

   nRow: Top pixel where calendar must be shown. If ommited Calendar will shown
         Centered.

   nCol: Same for left pixel.

   oDlg: Reference of the container object. It can be a Dialog or a Window.

Returns:
   A date Varible selected in calendar.


Notes:
   You can get relative position for nRow and nCol with function 
   GetCoors() as follows:

   aRect := GetCoors( oGet:hWnd )
   nRow := aRect[ 1 ]
   nCol := aRect[ 4 ]

   FwCalendar works over a resource dialog created with WorShop 4.5. This
   must be included in your *.rc or *.dll

Compile with:  /n /a /v

Author: Manuel Mercado,
        Freelance programmer.
        Salamanca, Gto., Mxico.

Version: 1.0  1/Sep/2001  FreeWare

Enjoy it,  It's Free !

*/

#include "Fivewin.ch"

//--------------------------------------------------------------------------//

Function FwCalendar( dDate, nRow, nCol, oWnd )

   Local nEle, nId, xVar, hBrush, cDateFormat, oRect, ;
         lOk := .F.

   Default nCol := 0

   If ValType( dDate ) != "D" .or. Empty( dDate )
      dDate := Date()
   EndIf

   cDateFormat := SET( _SET_DATEFORMAT, "dd/mm/yyyy" )

   PRIVATE cMonth,oDlg,cDate,;
           aObj := Array(7),;
           nLast := 1,;
           aCoords := {},;
           aSay := Array(37),;
           cSay := Array(37),;
           nYear := Year(dDate),;
           nMonth := Month(dDate),;
           nDay := DAY(dDate),;
           aMonth := {"January","February","March","April","May","June","July",;
                      "August","September","October","November","December"}
           aDay := {"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday",;
                    "Saturday"}

   If ValType( oWnd ) = "O"

      oRect := oWnd:GetRect()

      If ValType( nRow ) = "N"
         nRow += oRect:nTop
         nCol += oRect:nLeft
      EndIf

      If ( nEle := ( GetSysMetrics( 1 ) - ( nRow + 254 ) ) - 25 ) < 0
         nRow += nEle
      EndIf

      If ( nEle := ( GetSysMetrics( 0 ) - ( nCol + 158 ) ) ) < 0
         nCol += nEle
      EndIf

   EndIf

   cDate := aMonth[Month(dDate)] + Space( 1 ) + LTrim(Str(Day(dDate))) + ", " + ;
            LTrim(Str(Year(dDate)))

   nMonth := Month(dDate)
   cMonth := aMonth[nMonth]

   DEFINE FONT oFont NAME "MS Sans Serif" SIZE 0, -8

   DEFINE DIALOG oDlg NAME "FW_CALE" TITLE "FwCalendar" FONT oFont

   REDEFINE SAY aObj[1] VAR cDate ID 100 OF oDlg

   REDEFINE COMBOBOX aObj[2] VAR cMonth ID 101 ITEMS aMonth OF oDlg;
            ON CHANGE (nMonth := ASCAN(aMonth,cMOnth),fActMonth(aSay,cSay))

   REDEFINE GET aObj[3] VAR nYear ID 102 OF oDlg PICTURE "@K 9999";
            SPINNER ON UP (++nYear,aObj[3]:Refresh(),fActMonth(aSay,cSay),;
                           aObj[5]:SetFocus());
            ON DOWN (--nYear,aObj[3]:Refresh(),fActMonth(aSay,cSay),;
                     aObj[5]:SetFocus())

            aObj[3]:bGotFocus := {|x| x:Selectall()}
            aObj[3]:bLostFocus := {|x| fActMonth(aSay,cSay),aObj[5]:SetFocus()}

   REDEFINE LISTBOX aObj[5] VAR xVar ITEMS {} ID 105 OF oDlg
            aObj[5]:bKeyDown := {|nkey| fSetDay(nKey)}
            aObj[5]:bLostFocus := {|| aSay[nLast]:SetColor(CLR_WHITE,CLR_GRAY),;
                                      aSay[nLast]:Refresh() }
            aObj[5]:bGotFocus := {|| fActMonth(aSay,cSay) }

   REDEFINE SAY aObj[4] ID 104 OF oDlg COLOR CLR_WHITE,CLR_BLUE

   REDEFINE SAY aSay[1] VAR cSay[1] ID 201 OF oDlg
   REDEFINE SAY aSay[2] VAR cSay[2] ID 202 OF oDlg
   REDEFINE SAY aSay[3] VAR cSay[3] ID 203 OF oDlg
   REDEFINE SAY aSay[4] VAR cSay[4] ID 204 OF oDlg
   REDEFINE SAY aSay[5] VAR cSay[5] ID 205 OF oDlg
   REDEFINE SAY aSay[6] VAR cSay[6] ID 206 OF oDlg
   REDEFINE SAY aSay[7] VAR cSay[7] ID 207 OF oDlg
   REDEFINE SAY aSay[8] VAR cSay[8] ID 208 OF oDlg
   REDEFINE SAY aSay[9] VAR cSay[9] ID 209 OF oDlg
   REDEFINE SAY aSay[10] VAR cSay[10] ID 210 OF oDlg
   REDEFINE SAY aSay[11] VAR cSay[11] ID 211 OF oDlg
   REDEFINE SAY aSay[12] VAR cSay[12] ID 212 OF oDlg
   REDEFINE SAY aSay[13] VAR cSay[13] ID 213 OF oDlg
   REDEFINE SAY aSay[14] VAR cSay[14] ID 214 OF oDlg
   REDEFINE SAY aSay[15] VAR cSay[15] ID 215 OF oDlg
   REDEFINE SAY aSay[16] VAR cSay[16] ID 216 OF oDlg
   REDEFINE SAY aSay[17] VAR cSay[17] ID 217 OF oDlg
   REDEFINE SAY aSay[18] VAR cSay[18] ID 218 OF oDlg
   REDEFINE SAY aSay[19] VAR cSay[19] ID 219 OF oDlg
   REDEFINE SAY aSay[20] VAR cSay[20] ID 220 OF oDlg
   REDEFINE SAY aSay[21] VAR cSay[21] ID 221 OF oDlg
   REDEFINE SAY aSay[22] VAR cSay[22] ID 222 OF oDlg
   REDEFINE SAY aSay[23] VAR cSay[23] ID 223 OF oDlg
   REDEFINE SAY aSay[24] VAR cSay[24] ID 224 OF oDlg
   REDEFINE SAY aSay[25] VAR cSay[25] ID 225 OF oDlg
   REDEFINE SAY aSay[26] VAR cSay[26] ID 226 OF oDlg
   REDEFINE SAY aSay[27] VAR cSay[27] ID 227 OF oDlg
   REDEFINE SAY aSay[28] VAR cSay[28] ID 228 OF oDlg
   REDEFINE SAY aSay[29] VAR cSay[29] ID 229 OF oDlg
   REDEFINE SAY aSay[30] VAR cSay[30] ID 230 OF oDlg
   REDEFINE SAY aSay[31] VAR cSay[31] ID 231 OF oDlg
   REDEFINE SAY aSay[32] VAR cSay[32] ID 232 OF oDlg
   REDEFINE SAY aSay[33] VAR cSay[33] ID 233 OF oDlg
   REDEFINE SAY aSay[34] VAR cSay[34] ID 234 OF oDlg
   REDEFINE SAY aSay[35] VAR cSay[35] ID 235 OF oDlg
   REDEFINE SAY aSay[36] VAR cSay[36] ID 236 OF oDlg
   REDEFINE SAY aSay[37] VAR cSay[37] ID 237 OF oDlg

   For nEle := 1 To 37
      aSay[nEle]:Setcolor(CLR_BLACK,CLR_WHITE)
   Next

   REDEFINE BUTTON aObj[6] ID 301 OF oDlg ACTION (lOk := .T.,oDlg:End())
   REDEFINE BUTTON aObj[7] ID 302 OF oDlg ACTION oDlg:End()

   oDlg:bLclicked := {|nRow,nCol| aObj[ 5 ]:SetFocus(), fSetDay( nRow, nCol ) }
   oDlg:bGotFocus := {|| aObj[ 5 ]:SetFocus()}

   ACTIVATE DIALOG oDlg ;
            ON INIT ( AEval( aSay, {|e,n| AAdd( aCoords, ;
                      GetCoors(aSay[ n ]:hWnd ) ) } ), ;
                      fActMonth( aSay, cSay ),aObj[ 5 ]:SetFocus(), ;
                      If( ValType( nRow ) != "N", WndCenter(oDlg:hWnd), ;
                          oDlg:Move( nRow, nCol ) ) )

   If lOk
      dDate := CtoD( LTrim( Str( nDay ) ) + "/" + LTrim( Str( nMonth ) ) + ;
                     "/" + LTrim( Str( nYear ) ) )
   EndIf

   Set( _SET_DATEFORMAT, cDateFormat )

   oFont:End()

   If oRect != Nil
      DeleteObject( oRect )
   EndIf

Return dDate

//--------------------------------------------------------------------------//

Static Function fActMonth( aSay, cSay )

Local lActive,;
      dNewDate := CTOD("1/" + LTrim(Str(nMonth)) + "/" + Str(nYear,4)),;
      nDayNue := 1,;
      nLastDay := CTOD("1/" + LTrim(Str(nMonth)) + "/" +;
                       LTrim(Str(nYear))) + 32

nLastDay := nLastDay - Day(nLastDay)               
nLastDay := Day(nLastDay)
nDay := If(nDay > nLastDay,nLastDay,nDay)

AFILL(cSay,Space(2))
AEVAL(aSay,{|e,n| aSay[n]:Refresh()})

nEle := Dow(dNewDate)

lActive := aObj[5]:lFocused

While Month(dNewDate) == nMonth
   cSay[nEle] := Str(nDayNue++,2)
   If nDay = Day(dNewDate)
      aSay[nLast]:SetColor(CLR_BLACK,CLR_WHITE)
      aSay[nLast]:Refresh()
      aSay[nEle]:SetColor(CLR_WHITE,If(lActive,CLR_BLUE,CLR_GRAY))
      aSay[nEle]:Refresh()
      nLast := nEle
   Else
      aSay[nEle]:SetColor(CLR_BLACK,CLR_WHITE)
      aSay[nEle]:Refresh()
   EndIf
   ++nEle
   ++dNewDate
EndDo

dNewDate := CTOD(LTrim(Str(nDay)) + "/" + LTrim(Str(nMonth)) + "/" +;
           LTrim(Str(nYear)))
cDate := aMonth[Month(dNewDate)] + Space( 1 ) + LTrim(Str(Day(dNewDate))) + ", " + ;
         LTrim(Str(Year(dNewDate)))

aObj[1]:Refresh()

Return Nil

//--------------------------------------------------------------------------//

STATIC FUNCTION fSetDay( nRow, nCol )

Local dNewDate, nEle := 0

If ValType(nCol) = "N"
   AEVAL(aCoords,{|x,n| nEle := If(nRow >= x[1] .and. nCol >= x[2] .and.;
                        nRow <= x[3] .and. nCol <= x[4],n,nEle)})

   If nEle > 0 .and. VAL(cSay[nEle]) > 0
      aSay[nLast]:Setcolor(CLR_BLACK,CLR_WHITE)
      aSay[nLast]:Refresh()
      aSay[nEle]:Setcolor(CLR_WHITE,CLR_BLUE)
      aSay[nEle]:Refresh()
      nLast := nEle
      nDay := VAL(cSay[nEle])
   EndIf

Else
   Do Case
      Case nRow = VK_LEFT
         If nLast = 1 .or. Empty(cSay[nLast - 1])
            Return Nil
         EndIf
         aSay[nLast]:Setcolor(CLR_BLACK,CLR_WHITE)
         aSay[nLast]:Refresh()
         aSay[nLast - 1]:Setcolor(CLR_WHITE,CLR_BLUE)
         aSay[nLast - 1]:Refresh()
         nDay := VAL(cSay[--nLast])
      Case nRow = VK_RIGHT
         If nLast = 37 .or. Empty(cSay[nLast + 1])
            Return Nil
         EndIf
         aSay[nLast]:Setcolor(CLR_BLACK,CLR_WHITE)
         aSay[nLast]:Refresh()
         aSay[nLast + 1]:Setcolor(CLR_WHITE,CLR_BLUE)
         aSay[nLast + 1]:Refresh()
         nDay := VAL(cSay[++nLast])
      Case nRow = VK_UP
         If nLast < 8 .or. Empty(cSay[nLast - 7])
            Return Nil
         EndIf
         aSay[nLast]:Setcolor(CLR_BLACK,CLR_WHITE)
         aSay[nLast]:Refresh()
         aSay[nLast - 7]:Setcolor(CLR_WHITE,CLR_BLUE)
         aSay[nLast - 7]:Refresh()
         nDay := VAL(cSay[nLast -= 7])
      Case nRow = VK_DOWN
         If nLast > 35 .or. (nLast + 7) > 37 .or. Empty(cSay[nLast + 7])
            Return Nil
         EndIf
         aSay[nLast]:Setcolor(CLR_BLACK,CLR_WHITE)
         aSay[nLast]:Refresh()
         aSay[nLast + 7]:Setcolor(CLR_WHITE,CLR_BLUE)
         aSay[nLast + 7]:Refresh()
         nDay := VAL(cSay[nLast += 7])
   EndCase
EndIf

dNewDate := CTOD(LTrim(Str(nDay)) + "/" + LTrim(Str(nMonth)) + "/" +;
           LTrim(Str(nYear)))
cDate := aMonth[Month(dNewDate)] + Space( 1 ) + LTrim(Str(Day(dNewDate))) + ", " + ;
         LTrim(Str(Year(dNewDate)))

aObj[1]:Refresh()

Return Nil
